package error;

import java.text.MessageFormat;

import environment.Environment;

/**
 * Prints error messages and potentially terminates the application.
 */
public class ErrorManager {

    public static final int CRITICAL_LEVEL = 3;

    private static final String ANSI_RESET = "\u001B[0m";
    private static final String ANSI_RED = "\u001B[31m";
    private static final String ANSI_PURPLE = "\u001B[35m";

    private Environment environment;

    public ErrorManager() {
        this.environment = Environment.getInstance();
    }

    public void generateError(LispException lispException) {
        printError(lispException);

        if (isCritical(lispException))
            environment.terminateExceptionally();
    }

    private void printError(LispException lispException) {
        String formattedMessage = formatMessage(lispException);
        environment.getErrorOutput().println(formattedMessage);
    }

    private String formatMessage(LispException lispException) {
        String color = isCritical(lispException) ? ANSI_PURPLE : ANSI_RED;

        return MessageFormat.format("{0}error: {1}{2}", color, lispException.getMessage(), ANSI_RESET);
    }

    private boolean isCritical(LispException lispException) {
        return lispException.getSeverity() >= CRITICAL_LEVEL;
    }

}
